@@ -369,6 +369,25 @@ ev_tmp_uri_unlink (const gchar *uri)
 	g_object_unref (file);
 }
 
+gboolean
+ev_file_is_temp (GFile *file)
+{
+	gchar   *path;
+	gboolean retval;
+
+	if (!g_file_is_native (file))
+		return FALSE;
+
+	path = g_file_get_path (file);
+	if (!path)
+		return FALSE;
+
+	retval = g_str_has_prefix (path, g_get_tmp_dir ());
+	g_free (path);
+
+	return retval;
+}
+
 /**
  * ev_xfer_uri_simple:
  * @from: the source URI
@@ -397,9 +416,7 @@ ev_xfer_uri_simple (const char *from,
 	target_file = g_file_new_for_uri (to);
 	
 	result = g_file_copy (source_file, target_file,
-#if GLIB_CHECK_VERSION(2,19,0)
 			      G_FILE_COPY_TARGET_DEFAULT_PERMS |
-#endif
 			      G_FILE_COPY_OVERWRITE,
 			      NULL, NULL, NULL, error);
 
