@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -36,6 +36,7 @@
 #include "ev-document-print.h"
 #include "ev-document-annotations.h"
 #include "ev-document-attachments.h"
+#include "ev-document-text.h"
 #include "ev-debug.h"
 
 #include <errno.h>
@@ -49,6 +50,8 @@ static void ev_job_links_init             (EvJobLinks            *job);
 static void ev_job_links_class_init       (EvJobLinksClass       *class);
 static void ev_job_attachments_init       (EvJobAttachments      *job);
 static void ev_job_attachments_class_init (EvJobAttachmentsClass *class);
+static void ev_job_annots_init            (EvJobAnnots           *job);
+static void ev_job_annots_class_init      (EvJobAnnotsClass      *class);
 static void ev_job_render_init            (EvJobRender           *job);
 static void ev_job_render_class_init      (EvJobRenderClass      *class);
 static void ev_job_page_data_init         (EvJobPageData         *job);
@@ -91,6 +94,7 @@ static guint job_find_signals[FIND_LAST_SIGNAL] = { 0 };
 G_DEFINE_ABSTRACT_TYPE (EvJob, ev_job, G_TYPE_OBJECT)
 G_DEFINE_TYPE (EvJobLinks, ev_job_links, EV_TYPE_JOB)
 G_DEFINE_TYPE (EvJobAttachments, ev_job_attachments, EV_TYPE_JOB)
+G_DEFINE_TYPE (EvJobAnnots, ev_job_annots, EV_TYPE_JOB)
 G_DEFINE_TYPE (EvJobRender, ev_job_render, EV_TYPE_JOB)
 G_DEFINE_TYPE (EvJobPageData, ev_job_page_data, EV_TYPE_JOB)
 G_DEFINE_TYPE (EvJobThumbnail, ev_job_thumbnail, EV_TYPE_JOB)
@@ -434,6 +438,85 @@ ev_job_attachments_new (EvDocument *document)
 	return job;
 }
 
+/* EvJobAnnots */
+static void
+ev_job_annots_init (EvJobAnnots *job)
+{
+	EV_JOB (job)->run_mode = EV_JOB_RUN_THREAD;
+}
+
+static void
+ev_job_annots_dispose (GObject *object)
+{
+	EvJobAnnots *job;
+
+	ev_debug_message (DEBUG_JOBS, NULL);
+
+	job = EV_JOB_ANNOTS (object);
+
+	if (job->annots) {
+		g_list_foreach (job->annots, (GFunc)ev_mapping_list_unref, NULL);
+		g_list_free (job->annots);
+		job->annots = NULL;
+	}
+
+	G_OBJECT_CLASS (ev_job_annots_parent_class)->dispose (object);
+}
+
+static gboolean
+ev_job_annots_run (EvJob *job)
+{
+	EvJobAnnots *job_annots = EV_JOB_ANNOTS (job);
+	gint         i;
+
+	ev_debug_message (DEBUG_JOBS, NULL);
+	ev_profiler_start (EV_PROFILE_JOBS, "%s (%p)", EV_GET_TYPE_NAME (job), job);
+
+	ev_document_doc_mutex_lock ();
+	for (i = 0; i < ev_document_get_n_pages (job->document); i++) {
+		EvMappingList *mapping_list;
+		EvPage        *page;
+
+		page = ev_document_get_page (job->document, i);
+		mapping_list = ev_document_annotations_get_annotations (EV_DOCUMENT_ANNOTATIONS (job->document),
+									page);
+		g_object_unref (page);
+
+		if (mapping_list)
+			job_annots->annots = g_list_prepend (job_annots->annots, mapping_list);
+	}
+	ev_document_doc_mutex_unlock ();
+
+	job_annots->annots = g_list_reverse (job_annots->annots);
+
+	ev_job_succeeded (job);
+
+	return FALSE;
+}
+
+static void
+ev_job_annots_class_init (EvJobAnnotsClass *class)
+{
+	GObjectClass *oclass = G_OBJECT_CLASS (class);
+	EvJobClass   *job_class = EV_JOB_CLASS (class);
+
+	oclass->dispose = ev_job_annots_dispose;
+	job_class->run = ev_job_annots_run;
+}
+
+EvJob *
+ev_job_annots_new (EvDocument *document)
+{
+	EvJob *job;
+
+	ev_debug_message (DEBUG_JOBS, NULL);
+
+	job = g_object_new (EV_TYPE_JOB_ANNOTS, NULL);
+	job->document = g_object_ref (document);
+
+	return job;
+}
+
 /* EvJobRender */
 static void
 ev_job_render_init (EvJobRender *job)
@@ -461,7 +544,7 @@ ev_job_render_dispose (GObject *object)
 	}
 
 	if (job->selection_region) {
-		gdk_region_destroy (job->selection_region);
+		cairo_region_destroy (job->selection_region);
 		job->selection_region = NULL;
 	}
 
@@ -593,9 +676,17 @@ ev_job_page_data_run (EvJob *job)
 	ev_document_doc_mutex_lock ();
 	ev_page = ev_document_get_page (job->document, job_pd->page);
 
-	if ((job_pd->flags & EV_PAGE_DATA_INCLUDE_TEXT) && EV_IS_SELECTION (job->document))
+	if ((job_pd->flags & EV_PAGE_DATA_INCLUDE_TEXT_MAPPING) && EV_IS_DOCUMENT_TEXT (job->document))
 		job_pd->text_mapping =
-			ev_selection_get_selection_map (EV_SELECTION (job->document), ev_page);
+			ev_document_text_get_text_mapping (EV_DOCUMENT_TEXT (job->document), ev_page);
+	if ((job_pd->flags & EV_PAGE_DATA_INCLUDE_TEXT) && EV_IS_DOCUMENT_TEXT (job->document))
+		job_pd->text =
+			ev_document_text_get_text (EV_DOCUMENT_TEXT (job->document), ev_page);
+	if ((job_pd->flags & EV_PAGE_DATA_INCLUDE_TEXT_LAYOUT) && EV_IS_DOCUMENT_TEXT (job->document))
+		ev_document_text_get_text_layout (EV_DOCUMENT_TEXT (job->document),
+						  ev_page,
+						  &(job_pd->text_layout),
+						  &(job_pd->text_layout_length));
 	if ((job_pd->flags & EV_PAGE_DATA_INCLUDE_LINKS) && EV_IS_DOCUMENT_LINKS (job->document))
 		job_pd->link_mapping =
 			ev_document_links_get_links (EV_DOCUMENT_LINKS (job->document), ev_page);
@@ -840,6 +931,8 @@ ev_job_load_run (EvJob *job)
 	   because, e.g., a password is required - if so, just reload rather than
 	   creating a new instance */
 	if (job->document) {
+		const gchar *uncompressed_uri;
+
 		if (job_load->password) {
 			ev_document_security_set_password (EV_DOCUMENT_SECURITY (job->document),
 							   job_load->password);
@@ -848,9 +941,11 @@ ev_job_load_run (EvJob *job)
 		job->failed = FALSE;
 		job->finished = FALSE;
 		g_clear_error (&job->error);
-		
+
+		uncompressed_uri = g_object_get_data (G_OBJECT (job->document),
+						      "uri-uncompressed");
 		ev_document_load (job->document,
-				  job_load->uri,
+				  uncompressed_uri ? uncompressed_uri : job_load->uri,
 				  &error);
 	} else {
 		job->document = ev_document_factory_get_document (job_load->uri,
@@ -1083,7 +1178,7 @@ ev_job_find_dispose (GObject *object)
 		gint i;
 
 		for (i = 0; i < job->n_pages; i++) {
-			g_list_foreach (job->pages[i], (GFunc)g_free, NULL);
+			g_list_foreach (job->pages[i], (GFunc)ev_rectangle_free, NULL);
 			g_list_free (job->pages[i]);
 		}
 
