@@ -184,6 +184,15 @@ struct _EvView {
 	/* Annotations */
 	GList             *window_children;
 	EvViewWindowChild *window_child_focus;
+	EvMapping         *focus_annotation;
+	gboolean           adding_annot;
+	EvAnnotationType   adding_annot_type;
+
+	/* Synctex */
+	EvMapping *synctex_result;
+
+	/* Accessibility */
+	gboolean a11y_enabled;
 };
 
 struct _EvViewClass {
@@ -199,6 +208,10 @@ struct _EvViewClass {
 	void    (*popup_menu)		  (EvView         *view,
 					   GList          *items);
 	void    (*selection_changed)      (EvView         *view);
+	void    (*sync_source)            (EvView         *view,
+					   EvSourceLink   *link);
+	void    (*annot_added)            (EvView         *view,
+					   EvAnnotation   *annot);
 };
 
 void _get_page_size_for_scale_and_rotation (EvDocument *document,
