@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef EV_WINDOW_H
@@ -27,7 +27,6 @@
 #include <gtk/gtk.h>
 
 #include "ev-link.h"
-#include "ev-page-cache.h"
 
 G_BEGIN_DECLS
 
@@ -69,18 +68,25 @@ struct _EvWindowClass {
 	GtkWindowClass		base_class;
 };
 
-GType		ev_window_get_type	  (void);
-GtkWidget      *ev_window_new             (void);
-const char     *ev_window_get_uri         (EvWindow       *ev_window);
-void		ev_window_open_uri	  (EvWindow       *ev_window,
-					   const char     *uri,
-					   EvLinkDest     *dest,
-					   EvWindowRunMode mode,
-					   const gchar    *search_string);
-gboolean	ev_window_is_empty	  (const EvWindow *ev_window);
-void		ev_window_print_range     (EvWindow       *ev_window,
-					   int             first_page,
-					   int		   last_page);
+GType		ev_window_get_type	(void) G_GNUC_CONST;
+GtkWidget      *ev_window_new           (void);
+const char     *ev_window_get_uri       (EvWindow       *ev_window);
+void		ev_window_open_uri	(EvWindow       *ev_window,
+					 const char     *uri,
+					 EvLinkDest     *dest,
+					 EvWindowRunMode mode,
+					 const gchar    *search_string);
+void		ev_window_open_document (EvWindow       *ev_window,
+					 EvDocument     *document,
+					 EvLinkDest     *dest,
+					 EvWindowRunMode mode,
+					 const gchar    *search_string);
+gboolean	ev_window_is_empty	(const EvWindow *ev_window);
+void		ev_window_print_range   (EvWindow       *ev_window,
+					 int             first_page,
+					 int		 last_page);
+const gchar *	ev_window_get_dbus_object_path (EvWindow *ev_window);
+
 
 G_END_DECLS
 
